﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.ComposedBuild.Core
{
    public class ComposedBuildManager
    {
        public void Initialize(string configurationXml, string tfsCollectionUrl, string localWorkingDirectory)
        {
            if (String.IsNullOrEmpty(configurationXml))
                throw new ArgumentException("configurationXml is null or empty.", "configurationXml");
            if (String.IsNullOrEmpty(tfsCollectionUrl))
                throw new ArgumentException("tfsCollectionUrl is null or empty.", "tfsCollectionUrl");
            if (String.IsNullOrEmpty(localWorkingDirectory))
                throw new ArgumentException("localWorkingDirectory is null or empty.", "localWorkingDirectory");

            Configuration = new ComposedBuildConfiguration(configurationXml);

            LocalWorkingDirectory = localWorkingDirectory;
            TfsCollectionUrl = tfsCollectionUrl;

            if (Directory.Exists(localWorkingDirectory) == false)
            {
                throw new DirectoryNotFoundException(
                    String.Format("Directory '{0}' does not exist.", localWorkingDirectory));
            }

            InitializeDownloads();

            IsInitialized = true;
        }

        public bool IsInitialized
        {
            get;
            private set;
        }

        public ComposedBuildConfiguration Configuration { get; set; }
        public string LocalWorkingDirectory { get; set; }
        public string TfsCollectionUrl { get; set; }
        private void InitializeDownloads()
        {
            var session = new TfsSession(TfsCollectionUrl);

            BuildInfo info;

            foreach (var item in Configuration.Downloads)
            {
                info = session.GetBuildInfo(item.TeamProject,
                           item.BuildName,
                           item.BuildNumber);

                if (info == null)
                {
                    throw new InvalidOperationException(
                        String.Format("Unknown build.  (Team Project: '{0}', Build Name: '{1}', Build Number: '{2}')",
                        item.TeamProject, item.BuildName, item.BuildNumber));
                }

                item.InitializeForDownload(info.DropLocation, LocalWorkingDirectory);
            }
        }
        private void DirectoryCopy(string sourceDirName, string destDirName, bool copySubDirs)
        {
            // Get the subdirectories for the specified directory.
            DirectoryInfo dir = new DirectoryInfo(sourceDirName);
            DirectoryInfo[] dirs = dir.GetDirectories();

            if (!dir.Exists)
            {
                throw new DirectoryNotFoundException(
                    "Source directory does not exist or could not be found: "
                    + sourceDirName);
            }

            // If the destination directory doesn't exist, create it. 
            if (!Directory.Exists(destDirName))
            {
                Directory.CreateDirectory(destDirName);
            }

            // Get the files in the directory and copy them to the new location.
            FileInfo[] files = dir.GetFiles();
            foreach (FileInfo file in files)
            {
                string temppath = Path.Combine(destDirName, file.Name);
                file.CopyTo(temppath, false);
            }

            // If copying subdirectories, copy them and their contents to new location. 
            if (copySubDirs)
            {
                foreach (DirectoryInfo subdir in dirs)
                {
                    string temppath = Path.Combine(destDirName, subdir.Name);
                    DirectoryCopy(subdir.FullName, temppath, copySubDirs);
                }
            }
        }

        private void WriteMessage(string message)
        {
            Console.WriteLine(message);
        }

        public void Download()
        {
            WriteMessage("Starting downloads.");

            int totalDownloadCount = Configuration.Downloads.Count;

            WriteMessage(String.Format("Download count: {0}", totalDownloadCount));

            BuildOutput currentDownload = null;

            for (int i = 0; i < totalDownloadCount; i++)
            {
                currentDownload = Configuration.Downloads[i];

                WriteMessage(
                    String.Format("Starting download {0} of {1}: {2}",
                        i + 1,
                        totalDownloadCount,
                        currentDownload.ToString()));

                WriteMessage(String.Format("Downloading '{0}' to '{1}'...",
                    currentDownload.SourcePath, currentDownload.TargetPath));

                DirectoryCopy(currentDownload.SourcePath, currentDownload.TargetPath, true);

                WriteMessage(
                    String.Format("Completed download {0} of {1}: {2}",
                        i + 1,
                        totalDownloadCount,
                        currentDownload.ToString()));
            }


            WriteMessage("Downloads completed.");
        }

        protected string SafeTrim(string value)
        {
            if (value == null)
            {
                return String.Empty;
            }
            else
            {
                return value.Trim();
            }
        }
    }
}
