﻿using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Client;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.ComposedBuild.Core
{
    public class TfsSession
    {
        public TfsSession(string url)
        {
            if (String.IsNullOrEmpty(url))
                throw new ArgumentException("url is null or empty.", "url");

            _Url = url;

            Connect();
        }

        private string _Url;
        public string Url
        {
            get { return _Url; }
        }
        

        private TfsTeamProjectCollection m_Tpc;
        protected TfsTeamProjectCollection Tpc
        {
            get
            {
                if (m_Tpc == null)
                {
                    throw new InvalidOperationException(
                        "Team project collection has not been initialized.  Call Connect(string).");
                }

                return m_Tpc;
            }
        }

        protected void Connect()
        {
            Connect(_Url);
        }

        protected void Connect(string tfsUrl)
        {
            if (String.IsNullOrEmpty(tfsUrl))
                throw new ArgumentException("tfsUrl is null or empty.", "tfsUrl");

            Uri uri = new Uri(tfsUrl);

            m_Tpc = TfsTeamProjectCollectionFactory.GetTeamProjectCollection(uri);

            Tpc.Authenticate();
        }

        private IBuildServer _BuildServiceInstance;
        public IBuildServer BuildServiceInstance
        {
            get
            {
                if (_BuildServiceInstance == null)
                {
                    _BuildServiceInstance = Tpc.GetService<IBuildServer>();
                }

                return _BuildServiceInstance;
            }
        }
        public List<string> GetBuildDefinitions(string teamProjectName)
        {
            IBuildDefinitionSpec[] buildDefinitionSpec = 
            { 
                BuildServiceInstance.CreateBuildDefinitionSpec(teamProjectName)
            };

            var result = BuildServiceInstance.QueryBuildDefinitions(buildDefinitionSpec);

            if (result == null)
            {
                return new List<string>();
            }
            else
            {
                List<string> returnValue = new List<string>();

                foreach (var item in result)
                {
                    foreach (var def in item.Definitions)
                    {
                        returnValue.Add(def.Name);
                    }
                }

                return returnValue;
            }
        }

        public List<string> GetBuildsForDefinition(string teamProjectName, string buildName)
        {
            IBuildDetailSpec buildDetailSpec =
                BuildServiceInstance.CreateBuildDetailSpec(
                teamProjectName, buildName);
            
            var result = BuildServiceInstance.QueryBuilds(buildDetailSpec);

            if (result == null)
            {
                return new List<string>();
            }
            else
            {
                List<string> builds = new List<string>();

                foreach (var build in result.Builds)
                {
                    builds.Add(build.BuildNumber);
                }

                return builds;
            }
        }
        public BuildInfo GetBuildInfo(string teamProjectName, string buildName, string buildNumber)
        {
            IBuildDetailSpec buildDetailSpec =
                BuildServiceInstance.CreateBuildDetailSpec(
                teamProjectName, buildName);
            
            var result = BuildServiceInstance.QueryBuilds(buildDetailSpec);

            if (result == null)
            {
            	return null;
            }
            else
            {
            	var builds = result.Builds;

                var match = (from temp in builds
                             where
                             String.Compare(temp.BuildNumber, buildNumber, true) == 0
                             select temp).FirstOrDefault();

                if (match == null)
                {
                    return null;
                }
                else
                {
                    return new BuildInfo(
                        match.BuildNumber, 
                        GetBuildDefinitionNameNullSafe(match), 
                        match.DropLocation);
                }
            }
             
        }
        private string GetBuildDefinitionNameNullSafe(IBuildDetail build)
        {
            if (build == null || build.BuildDefinition == null)
            {
                return String.Empty;
            }
            else
            {
                return build.BuildDefinition.Name;
            }
        }
    }
}