using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.ComposedBuild.Core;
using System.Collections.Generic;
using System.IO;
using System.Xml.Linq;

namespace Benday.ComposedBuild.Tests
{
    [TestClass]
    public class ComposedBuildManagerFixture : UnitTestBase
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
        }

        private ComposedBuildManager _SystemUnderTest;
        public ComposedBuildManager SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new ComposedBuildManager();
                }

                return _SystemUnderTest;
            }
        }

        private string _LocalWorkingDirectory;
        public string LocalWorkingDirectory
        {
            get
            {
                if (_LocalWorkingDirectory == null)
                {
                    _LocalWorkingDirectory =
                        Path.Combine(@"c:\temp\ComposedBuildManagerFixture\",
                        DateTime.Now.Ticks.ToString());

                    if (Directory.Exists(_LocalWorkingDirectory) == false)
                    {
                        Console.WriteLine("Creating local working directory at '{0}'", _LocalWorkingDirectory);
                        Directory.CreateDirectory(_LocalWorkingDirectory);
                    }
                }

                return _LocalWorkingDirectory;
            }
        }

        [TestMethod]
        public void LocalWorkingDirectoryIsCreatedAsPartOfTestAndIsNonBlank()
        {
            Assert.AreNotEqual<string>(String.Empty, LocalWorkingDirectory, "LocalWorkingDirectory should not be empty.");
            Assert.IsTrue(
                Directory.Exists(LocalWorkingDirectory),
                "LocalWorkingDirectory has not been created on disk at '{0}'.", LocalWorkingDirectory);
        }

        [TestMethod]
        public void IsInitializedIsFalseWhenInitializeHasNotBeenCalled()
        {
            Assert.IsFalse(SystemUnderTest.IsInitialized, "Value is wrong.");
        }

        private void CallInitializeWithValidValues()
        {
            SystemUnderTest.Initialize(
                            Resource1.ConfigForBuildOutputDownloadManagerFixture,
                            base.TfsCollectionUrl,
                            LocalWorkingDirectory);
        }

        [TestMethod]
        public void IsInitializedIsTrueWhenInitializeHasBeenCalled()
        {
            CallInitializeWithValidValues();

            Assert.IsTrue(SystemUnderTest.IsInitialized, "Value is wrong.");
        }

        [TestMethod]
        [ExpectedException(typeof(DirectoryNotFoundException))]
        public void InitializeThrowsExceptionWhenLocalWorkingDirectoryDoesNotExist()
        {
            SystemUnderTest.Initialize(
                            Resource1.ConfigForBuildOutputDownloadManagerFixture,
                            base.TfsCollectionUrl,
                            @"c:\bogus-dir");
        }

        [TestMethod]
        public void ConfigurationIsPopulatedAfterInitializeHasBeenCalled()
        {
            CallInitializeWithValidValues();

            Assert.IsNotNull(SystemUnderTest.Configuration, "Value is null.");
        }

        [TestMethod]
        public void LocalWorkingDirectoryIsPopulatedAfterInitializeHasBeenCalled()
        {
            CallInitializeWithValidValues();

            Assert.AreEqual<string>(LocalWorkingDirectory, SystemUnderTest.LocalWorkingDirectory, "Value is wrong.");
        }

        [TestMethod]
        public void TfsCollectionUrlIsPopulatedAfterInitializeHasBeenCalled()
        {
            CallInitializeWithValidValues();

            Assert.AreEqual<string>(TfsCollectionUrl, SystemUnderTest.TfsCollectionUrl, "Value is wrong.");
        }

        [TestMethod]
        public void DownloadersIsPopulatedAfterInitializeHasBeenCalled()
        {
            CallInitializeWithValidValues();

            foreach (var item in SystemUnderTest.Configuration.Downloads)
            {
                Assert.IsTrue(item.HasBeenInitializedForDownload);
            }
        }

        private void AssertDirectoryIsEmpty(string path)
        {
            var dir = new DirectoryInfo(path);

            Assert.IsTrue(dir.Exists, "Local dir doesn't exist.");
            Assert.AreEqual<int>(0, dir.GetDirectories().Length, "Shouldn't be any subfolders.");
            Assert.AreEqual<int>(0, dir.GetFiles().Length, "Shouldn't be any files.");
        }

        private void AssertDirectoryIsNotEmpty(string path)
        {
            var dir = new DirectoryInfo(path);

            Assert.IsTrue(dir.Exists, "Local dir doesn't exist.");
            Assert.AreNotEqual<int>(0, dir.GetDirectories().Length, "There should be subfolders.");
            Assert.AreEqual<int>(0, dir.GetFiles().Length, "Shouldn't be any files in the root of working dir.");
        }

        [TestMethod]
        [Timeout(30000)]
        public void DownloadToDisk()
        {
            CallInitializeWithValidValues();

            AssertDirectoryIsEmpty(LocalWorkingDirectory);

            SystemUnderTest.Download();

            AssertDirectoryIsNotEmpty(LocalWorkingDirectory);
        }
    }
}
